import re
from collections import OrderedDict

inFile  = "LS.txt"
outFile = "outFile.txt"
keySpecies = "ISHI-4"

oneLineLength = 90

def readFasta_dict(InfileNameFN):
    Infile = open(InfileNameFN, "r")
    recsFN = OrderedDict()
    for Line in Infile:
        Line = Line.rstrip("\n")
        if Line[0] == ">":
            Name            = Line
            recsFN[Name] = ""
        else:
            recsFN[Name] += Line
    Infile.close()
    return recsFN

def whiteSpaceAdd(recsFN1):
    longestNameLen = len(max(recsFN1.keys(), key = len))
    recsFN2        = OrderedDict()
    for name,value in recsFN1.items():
        nameWhiteSpace = name[1:] + " " * (longestNameLen - len(name) + 2)
        recsFN2[nameWhiteSpace] = value
    return recsFN2

def fasInterleavedFileMaker(oneLineLengthFN, recsFN, outFileFN):
    siteNum  = 1
    startPos = 0
    stopPos  = 0
    sequenceLength = len(list(recsFN.values())[0])
    out = open(outFileFN, "w")
    out.write(str(len(recsFN)) + " " + str(sequenceLength) + "\n")
    for i in range(sequenceLength):
        if i > 1 and i % oneLineLengthFN == 0:
            startPos = i - oneLineLengthFN
            stopPos = startPos + oneLineLengthFN
            #print("siteNum :", startPos + 1)
            #print("startPos:", startPos)
            #print("stopPos :", stopPos)
            #for j in range(startPos, stopPos):
            #    print(str(j) + ",", end="")
            #print()
            for name in recsFN.keys():
                out.write(name)
                for j in range(startPos, stopPos):
                    out.write(recsFN[name][j])
                out.write("\n")
            out.write("\n")
    
    #print("siteNum :", stopPos + 1)
    #print("startPos:", stopPos)
    #print("stopPos :", sequenceLength)
    #for i in range(stopPos, sequenceLength):
    #    print(str(i) + ",", end="")
    #print()
    for name in recsFN.keys():
        out.write(name)
        for i in range(stopPos, sequenceLength):
            out.write(recsFN[name][i])
        out.write("\n")
    out.close()

def keySpeciesPicker(keySpeciesFN, recsFN):
    r = re.compile(">" + keySpeciesFN)
    hitNames = [l for l in recsFN.keys() if r.match(l)]
    if len(hitNames) > 1:
        print("More than 2 keySpecies:%s. Stopped." % keySpeciesFN)
        exit()
    elif len(hitNames) == 0:
        print("No keySpecies:%s. Stopped." % keySpeciesFN)
        exit()
    else:
        return hitNames[0], recsFN[hitNames[0]]

def matchFirster (keyNameFN, keySequenceFN, recsFN):
    recsFN1 = OrderedDict()
    for name, value in recsFN.items():
        sequence = ""
        if name == keyNameFN:
            for i in range(len(keySequenceFN)):
                sequence += value[i]
        else:
            for i in range(len(keySequenceFN)):
                if keySequenceFN[i] == "X" or keySequenceFN[i] == "-" or keySequenceFN[i] == "N":
                    sequence += value[i]
                elif keySequenceFN[i] == value[i]:
                    sequence += "."
                else:
                    sequence += value[i]
        recsFN1[name] = sequence
    return recsFN1

recs = readFasta_dict(inFile)
keyName, keySequence = keySpeciesPicker(keySpecies, recs)
recs = matchFirster(keyName, keySequence, recs)
recs = whiteSpaceAdd(recs)
fasInterleavedFileMaker(oneLineLength, recs, outFile)
